require 'parallel_batch_api/builder'
require 'parallel_batch_api/processor'
require 'parallel_batch_api/configuration'

module ParallelBatchApi
  extend ParallelBatchApi::Configuration

  HTTP_METHOD_GET = 'GET'.freeze
  HTTP_METHOD_POST = 'POST'.freeze
  DEFAULT_BATCH_PATH = '/api/batch'.freeze

  class Middleware
    include ParallelBatchApi::Builder
    include ParallelBatchApi::Processor

    def initialize(app)
      @app = app
    end

    def call(env)
      if batch_request?(env)
        process(env)
      else
        @app.call(env)
      end
    end

    def batch_request?(env)
      valid_path?(env) && valid_method?(env)
    end

    def valid_path?(env)
      batch_path.include?(env['PATH_INFO'])
    end

    def valid_method?(env)
      env['REQUEST_METHOD'] == HTTP_METHOD_POST
    end

    def batch_path
      [ParallelBatchApi.batch_path]
    end
  end

  def self.batch_path
    super || DEFAULT_BATCH_PATH
  end
end
